function ajax(params) {
  if (typeof params === 'undefined') {
    params = {};
  }
  var request;
  var post_data;
  var headers = '';
  var timeout = params.timeout || 0;
  var headers_set = params.headers || {};
  var data = params.data || {};
  var url = params.url || '';
  var method = params.method || 'GET';
  var responseType = params.responseType || 'json';
  var callback = {
    init: params.onInit,
    loading: params.onLoading,
    success: params.onSuccess,
    error: params.onError,
    complete: params.onComplete,
    progress: params.onProgress
  }


  var try_again_timeout = false;
  var try_again_responseType = false;


  request = new XMLHttpRequest();
  if (request.timeout !== 0) {
    try {
      request.timeout = timeout;
    } catch (err) {
      try_again_timeout = true;
    }
  }
  try {
    request.responseType = responseType;
  } catch (err) {
    try_again_responseType = true;
  }
  if (request.onprogress !== undefined) {
    request.onprogress = function(e) {
      if (typeof callback.progress === 'function')
        callback.progress(e, this);
    };
  }
  request.onreadystatechange = function(e) {
    switch (this.readyState) {
      case 1:// init
        if (typeof callback.init === 'function')
          callback.init(this);
        break;

      case 2:// headers
        if (request !== null) {
          headers = request.getAllResponseHeaders();
        }
        break;

      case 3:// loading
        if (typeof callback.loading === 'function')
          callback.loading(this, headers);
        break;

      case 4:// complete
        if (this.status === 200) {
          if (typeof callback.success === 'function') {
            if (request.responseType === '') {
              if (responseType === 'json')
                callback.success(JSON.parse(this.responseText), headers, this);
              else
                callback.success(this.responseText, headers, this);
            } else {
              if (responseType === 'text')
                callback.success(this.responseText, headers, this);
              else
                callback.success(this.response, headers, this);
            }
          }
        } else {
          if (typeof callback.error === 'function')
            callback.error(this.status, this);
        }
        if (typeof callback.complete === 'function')
          callback.complete(this.status, this);
        break;
    }
  }


  switch (method) {
    case 'GET':
      request.open("GET", url + '?' + serialize(params.data), true);
      trySetOptions(try_again_timeout, try_again_responseType);
      setHeaders(headers_set);
      request.send();
      break;
    case 'POST':
      request.open("POST", url, true);
      trySetOptions(try_again_timeout, try_again_responseType);
      request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
      setHeaders(headers_set);
      request.send(serialize(data));
      break;
  }


  function setHeaders(values) {
    if (typeof values === "object") {
      for (p in values) {
        request.setRequestHeader(p, values[p]);
      }
    }
  }


  function trySetOptions(timeout, r_type) {
    if (timeout) {
      try {
        request.timeout = timeout;
      } catch (err) {
        //
      }
    }
    if (r_type) {
      try {
        request.responseType = responseType;
      } catch (err) {
        //
      }
    }
  }


  function serialize(info) {
    var data = info || "";
    var t = [];
    for (p in data) {
      t.push(p + '=' + data[p]);
    }
    return t.join('&');
  }
}
