document.addEventListener("DOMContentLoaded", documentReady);

var animate = (function(t) {
  var c = 0;
  var frame = 16;
  var frames = t / frame
  return function(callback) {
    var progress = c / t;
    if (c > t) {
      callback(1);
      c = 0;
      return;
    }
    callback(progress);
    c += frame;
    setTimeout(function() {
      animate(callback);
    }, frame);
  }
})(500);


function documentReady() {
  var input_ip;
  var input_port;
  var bt_submit;
  var form_socket;
  var notifications;
  var translations;

  input_ip = document.getElementById('ip');
  input_port = document.getElementById('port');
  bt_submit = document.getElementById('bt_submit');
  form_socket = document.getElementById('form_socket');
  notifications = document.getElementById('notifications');

  browser.storage.local.get("himp").then(function(options) {
    options = options.himp || {
      ip: "127.0.0.1",
      port: 8999,
      notifications: true
    };
    input_ip.value = options.ip;
    port.value = options.port;
    notifications.checked = options.notifications;
  }, function() {
    input_ip.value = "127.0.0.1";
    input_port.value = 8999;
    notifications.value = true;
  });

  form_socket.addEventListener('submit', function(e) {
    e.preventDefault();
    browser.storage.local.set({
      himp: {
        ip: input_ip.value,
        port: input_port.value,
        notifications: notifications.checked
      }
    }).then(function() {
//       browser.runtime.reload();
      input_ip.style.backgroundColor = 'rgba(255, 127, 42, 1)';
      input_port.style.backgroundColor = 'rgba(255, 127, 42, 1)';
      animate(function(progress) {
        input_ip.style.backgroundColor = 'rgba(255, 127, 42, ' + (1 - progress) + ')';;
        input_port.style.backgroundColor = 'rgba(255, 127, 42, ' + (1 - progress) + ')';;
      });
    });
  });


  translations = {
    diffSocket: browser.i18n.getMessage("optionsSpecifyDifferentSocket"),
    ipAddress: browser.i18n.getMessage("optionsIpAddress"),
    ipAddressPlaceholder: browser.i18n.getMessage("optionsIpAddressPlaceholder"),
    port: browser.i18n.getMessage("optionsPort"),
    optionsPortPlaceholder: browser.i18n.getMessage("optionsPortPlaceholder"),
    notifications: browser.i18n.getMessage("optionsNotifications"),
    save: browser.i18n.getMessage("optionsSave"),
    test: 'qqqqqq'
  }
  templateFill("template", translations);

}


function templateFill(class_name, i18n) {
  var template = document.getElementsByClassName(class_name);
  for (element in template) {
    if (typeof template[element] === "object") {
      if (template[element].dataset.replace !== undefined) {
        template[element].innerText = i18n[template[element].dataset.replace];
      }
      if (template[element].dataset.replace_placeholder !== undefined) {
        template[element].placeholder = i18n[template[element].dataset.replace_placeholder];
      }
      if (template[element].dataset.replace_value !== undefined) {
        template[element].value = i18n[template[element].dataset.replace_value];
      }
      if (template[element].dataset.replace_title !== undefined) {
        template[element].title = i18n[template[element].dataset.replace_title];
      }
    }
  }
}
